package jemu.core.device.tape;

/**
 * WAV file output manager
 * Converted from "TZX to WAV Converter v0.2 (C) 2006 Francisco Javier Crespo <tzx2wav@ya.com>"
 * 
 * Nice WAV format info in this web site:
 * http://ccrma.stanford.edu/courses/422/projects/WaveFormat/
 * 
 * @author John Girvin
 *
 */
public class CDT2WAVOutput extends CDT2WAVOut {
	//
	// INSTANCE VARIABLES
	//
	private WAVHeader wavHeader;
	private int bufPos = 0;
    public static boolean compile;

	/**
	 * Inner class to represent WAV file header data
	 */
	private class WAVHeader {
		// header data members
		public int   ChunkID;
		public int   ChunkSize;
		public int   Format;
		public int   fmtChunkID;
		public int   fmtChunkSize;
		public short AudioFormat;
		public short NumChannels;
		public int   SampleRate;
		public int   ByteRate;
		public short BlockAlign;
		public short BitsPerSample;
		public int   dataChunkID;
		public int   dataChunkSize;
		
		/**
		 * Get the length in bytes of this WAV header.
		 * @return length in bytes of this WAV header
		 */
		public int getLength() {
			return 44;
		}
		
		private int writeInt(byte[] buf, int pos, int data) {
			buf[pos++] = (byte) (data & 0xff);
			buf[pos++] = (byte) ((data >>  8) & 0xff);
			buf[pos++] = (byte) ((data >> 16) & 0xff);
			buf[pos++] = (byte) ((data >> 24) & 0xff);
			return 4;
		}
		
		private int writeShort(byte[] buf, int pos, short data) {
			buf[pos++] = (byte) (data & 0xff);
			buf[pos++] = (byte) ((data >>  8) & 0xff);
			return 2;
		}
		
		/**
		 * Write header content to a byte array
		 * @param buf
		 * @param pos
		 */
		public void write(byte[] buf, int pos) {
			pos += writeInt(buf, pos, ChunkID);
			pos += writeInt(buf, pos, ChunkSize);
			pos += writeInt(buf, pos, Format);
			pos += writeInt(buf, pos, fmtChunkID);
			pos += writeInt(buf, pos, fmtChunkSize);
			pos += writeShort(buf, pos, AudioFormat);
			pos += writeShort(buf, pos, NumChannels);
			pos += writeInt(buf, pos, SampleRate);
			pos += writeInt(buf, pos, ByteRate);
			pos += writeShort(buf, pos, BlockAlign);
			pos += writeShort(buf, pos, BitsPerSample);
			pos += writeInt(buf, pos, dataChunkID);
			pos += writeInt(buf, pos, dataChunkSize);
		}
	}
	
	//
	// CONSTRUCTOR
	//
	public CDT2WAVOutput(byte[] buf, int freq) {
		super(buf, freq);
	}
	
	/**
	 * Prepare for output
	 */
	protected void init() {
		// Initialise WAV file header for 8-bit mono output 95 26 CD 00
		wavHeader = new WAVHeader();
		wavHeader.ChunkID = 0x46464952;      // "RIFF" ID
		wavHeader.ChunkSize = 0xCD2695;
		wavHeader.Format = 0x45564157;       // "WAVE" ID
		wavHeader.fmtChunkID = 0x20746D66;   // "fmt " ID
		wavHeader.fmtChunkSize = 16;
		wavHeader.AudioFormat = 1;           // PCM Linear Quantization
		wavHeader.SampleRate = (int) getFrequency();
		wavHeader.dataChunkID = 0x61746164;  // "data" ID
		wavHeader.dataChunkSize = 0;
		wavHeader.NumChannels = 1;
		wavHeader.BitsPerSample = 8;
		wavHeader.ByteRate = (int) getFrequency();
		wavHeader.BlockAlign = 1;
        if (compile)
		wavHeader.write(getBuf(), 0);
		
		// Initialise buffer
		bufPos = wavHeader.getLength();
	}

	/**
	 * Generate WAV data for "numsamples" samples.
	 * @param numsamples
	 */
	protected void write(int numsamples) {
		// Update data length in WAV header
		wavHeader.dataChunkSize += numsamples;

		// Write samples at current amplitude
		byte sample = (isLowAmp() ? (byte) 0x26 : (byte) 0xDA);
		byte[] buf  = getBuf();

		for (int i = 0; i < numsamples; i++) {
            if (compile)
			buf[bufPos] = sample;
            bufPos++;
		}
	}

	/**
	 * Finalise output.
	 */
	protected void stop() {
		// Write WAV header to the start of the buffer
        if (compile)
		wavHeader.write(getBuf(), 0);
        jemu.system.cpc.CPC.recordcount = bufPos;
	}

}
